﻿/*
 *  TOPPERS ECHONET Lite Communication Middleware
 * 
 *  Copyright (C) 2015 Cores Co., Ltd. Japan
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id$
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Html;
using jQueryApi;
using System.Text;
using System.Collections;
using System.Text.RegularExpressions;

public class CodeGen
{
	public static CodeGen Page;
	public static Information Info;

	public static void Main()
	{
		if (Page == null) {
			Page = new CodeGen();
			Info = new Information();
		}

		jQuery.OnDocumentReady(() =>
		{
			Page.InitClassGroups(jQuery.Select("#index")[0]);
		});
	}

	public static Element GetPage(Element ele)
	{
		var page = ele;
		for (; ; ) {
			var attr = page.Attributes["data-role"];
			if (attr != null && attr.Value == "page")
				return page;

			page = page.ParentElement;
			if (page == null)
				break;
		}
		return null;
	}

	List<JsonNodeInfo> NodeList = new List<JsonNodeInfo>();
	JsonClassGroupInfo[] ClassGroups;
	JsonSkeltonResult WorkArea;
	JsonNodeInfo _currentNode;
	JsonObjectInfo _currentObject;
	JsonClassInfo NodeProfileClass;
	JsonClassInfo CurrentClass;
	public string MainH;
	public string MainC;
	public string MainCfg;

	public JsonNodeInfo CurrentNode
	{
		get { return _currentNode; }
		set
		{
			_currentNode = value;
			jQuery.Select("#ti_node_name")[0].SetValue((value != null) ? value.identifer : "");
			bool disabled = value == null;
			jQuery.Select("#bt_change_node")[0].SetDisabled(disabled);
			jQuery.Select("#bt_delete_node")[0].SetDisabled(disabled);
		}
	}

	public JsonObjectInfo CurrentObject
	{
		get { return _currentObject; }
		set
		{
			_currentObject = value;
			jQuery.Select("#ti_object_name")[0].SetValue((value != null) ? value.identifer : "");
			bool disabled = value == null;
			jQuery.Select("#bt_change_object")[0].SetDisabled(disabled);
			jQuery.Select("#bt_delete_object")[0].SetDisabled(disabled);
		}
	}

	void InitNodeList(Element ele)
	{
		var page = GetPage(ele);
		var info = jQuery.Select("#node_list", page);
		if (info == null)
			return;

		var content = "";
		foreach (var item in NodeList) {
			content += "<div data-role=\"collapsible\" data-inset=\"true\" " +
				"id=\"cl_node_" + item.identifer + "\"><h2>"
				+ item.identifer + "</h2><ul id=\"lv_node_" + item.identifer
				+ "\" data-role=\"listview\"></ul></div>";
		}
		info.Append(content).CollapsibleSet("refresh");
		foreach (var item in NodeList) {
			var nodeItem = jQuery.Select("#cl_node_" + item.identifer);
			jQuery.Select("a", nodeItem).Click(new JsDictionary("node", item), UpdateObjects);
		}
	}

	public bool AddNode(Element ele)
	{
		var page = GetPage(ele);
		var info = jQuery.Select("#node_list", page);
		if (info == null)
			return false;

		var name = jQuery.Select("#ti_node_identifer")[0].GetValue();
		if (!(new Regex("^[A-Za-z_][0-9A-Za-z_]+$")).Test(name)) {
			IdentiferNotInputError();
			return false;
		}

		if (CheckIdentifer(name)) {
			InvalidIdentiferError();
			return false;
		}

		var attr = "";
		if (jQuery.Select("#rc_node_attr_local")[0].GetChecked())
			attr = "local";
		else if (jQuery.Select("#rc_node_attr_sync")[0].GetChecked())
			attr = "sync";
		else if (jQuery.Select("#rc_node_attr_async")[0].GetChecked())
			attr = "async";

		var node = new JsonNodeInfo(NodeProfileClass, name, attr);
		NodeList.Add(node);
		var content = "<div data-role=\"collapsible\" data-inset=\"true\" " +
			"id=\"cl_node_" + node.identifer + "\"><h2>"
			+ node.identifer + "</h2><ul id=\"lv_node_" + node.identifer
			+ "\" data-role=\"listview\"></ul></div>";

		info.Append(content).CollapsibleSet("refresh");
		var nodeItem = jQuery.Select("#cl_node_" + node.identifer);
		jQuery.Select("a", nodeItem).Click(new JsDictionary("node", node), UpdateObjects);

		foreach (var item in node.type.properties) {
			if ((item.required.Count > 0) && (item.required[0] != "NONE")) {
				node.properties.Add(new JsonPropertyInfo(item));
			}
		}

		nodeItem.Collapsible("expand");

		UpdateObjects(node, ele);

		return true;
	}

	void UpdateObjects(jQueryEvent ev)
	{
		UpdateObjects((JsonNodeInfo)ev.Data["node"], ev.Target);
	}

	public bool ChangeNode(Element ele)
	{
		var page = GetPage(ele);

		var node = CurrentNode;
		if (node == null)
			return false;

		var info = jQuery.Select("#node_list", page);
		if (info == null)
			return false;

		var name = jQuery.Select("#ti_node_identifer")[0].GetValue();
		if (!(new Regex("^[A-Za-z_][0-9A-Za-z_]+$")).Test(name)) {
			IdentiferNotInputError();
			return false;
		}

		if (node.identifer != name && CheckIdentifer(name)) {
			InvalidIdentiferError();
			return false;
		}

		var attr = "";
		if (jQuery.Select("#rc_node_attr_local")[0].GetChecked())
			attr = "local";
		else if (jQuery.Select("#rc_node_attr_sync")[0].GetChecked())
			attr = "sync";
		else if (jQuery.Select("#rc_node_attr_async")[0].GetChecked())
			attr = "async";

		var nodeItem = jQuery.Select("#cl_node_" + node.identifer, info);
		nodeItem[0].Id = "cl_node_" + name;
		jQuery.Select("a", nodeItem).Text(name);
		jQuery.Select("#lv_node_" + node.identifer, nodeItem)[0].Id = "lv_node_" + name;

		info.CollapsibleSet("refresh");

		node.identifer = name;
		node.attribute = attr;

		return true;
	}

	public bool DeleteNode(Element ele)
	{
		var page = GetPage(ele);

		var node = CurrentNode;
		if (node == null)
			return false;

		var info = jQuery.Select("#node_list", page);
		if (info == null)
			return false;

		jQuery.Select("#cl_node_" + node.identifer, info).Remove();
		info.CollapsibleSet("refresh");

		CurrentNode = null;

		NodeList.Remove(node);

		if (NodeList.Count > 0)
			CurrentNode = NodeList[0];

		if (CurrentNode != null) {
			var nodeItem = jQuery.Select("#cl_node_" + node.identifer);
			nodeItem.Collapsible("expand");

			UpdateObjects(CurrentNode, ele);
		}

		return true;
	}

	void UpdateObjects(JsonNodeInfo node, Element ele)
	{
		CurrentNode = node;

		jQuery.Select("#ti_node_identifer")[0].SetValue(node.identifer);

		var attr = node.attribute;
		jQuery.Select("#rc_node_attr_local")[0].SetChecked(attr == "local");
		jQuery.Select("#rc_node_attr_sync")[0].SetChecked(attr == "sync");
		jQuery.Select("#rc_node_attr_async")[0].SetChecked(attr == "async");

		jQuery.Select("#rc_node_attr_local").CheckboxRadio("refresh");
		jQuery.Select("#rc_node_attr_sync").CheckboxRadio("refresh");
		jQuery.Select("#rc_node_attr_async").CheckboxRadio("refresh");

		var info = jQuery.Select("#lv_node_" + node.identifer, ele);
		if (info == null)
			return;

		var content = "";
		foreach (var iobj in node.objects) {
			content += "<li id=\"li_object_" + node.identifer + "\"><a href=\"#\">"
				+ node.identifer + "</a></li>";
		}
		info.Children().ReplaceWith("");
		info.Append(content).ListView().ListView("refresh");

		UpdateProperties(node);
	}

	void InitClassGroups(Element ele)
	{
		var page = GetPage(ele);

		jQuery.Select("#main_tabs").Tabs("option", "active", 1);

		jQuery.Select("#result_dialog").On("popupafteropen", null, null, QuerySkeleton);

		var info = jQuery.Select("#class_group_list", page);
		if (info == null)
			return;

		jQuery.Ajax(new jQueryAjaxOptions()
		{
			Url = "devices/ClassGroupList.json",
			Success = (data, textStatus, request) =>
			{
				JsonClassGroupInfo profileClass = null;
				var content = "";
				var _classGroups = (dynamic[])((data.GetType() == typeof(string)) ? jQuery.ParseJson((string)data) : data);
				var classGroups = new List<JsonClassGroupInfo>();
				foreach (var _item in _classGroups) {
					var item = new JsonClassGroupInfo(_item);
					classGroups.Add(item);
					if (item.classGroupCode == 0xE) {
						profileClass = item;
						continue;
					}
					else if (item.classGroupCode == 0xF)
						continue;

					content += "<div data-role=\"collapsible\" data-inset=\"true\" "
						+ "id=\"cl_class_group_" + item.classGroupCode.ToString("X2") + "\"><h2>"
						+ item.description + "</h2><ul id=\"lv_class_group_" + item.classGroupCode.ToString("X2")
						+ "\" data-role=\"listview\"></ul></div>";
				}
				info.Append(content).CollapsibleSet("refresh");
				foreach (var item in classGroups) {
					jQuery.Select("#cl_class_group_" + item.classGroupCode.ToString("X2")).On("updatelayout", null, new JsDictionary("clsgrp", item), UpdateClasses);
				}
				ClassGroups = classGroups.ToArray();

				if (profileClass != null) {
					jQuery.Ajax(new jQueryAjaxOptions()
					{
						Url = "devices/ClassList" + profileClass.classGroupCode.ToString("X2").ToUpper() + ".json",
						Success = (data1, textStatus1, request1) =>
						{
							var _classes = (dynamic[])((data1.GetType() == typeof(string)) ? jQuery.ParseJson((string)data1) : data1);
							var classes = new List<JsonClassInfo>();
							foreach (var _item in _classes) {
								var item = new JsonClassInfo(_item);
								classes.Add(item);
								item.classGroup = profileClass;
								if (item.classCode == 0xF0) {
									NodeProfileClass = item;
									CurrentNode = new JsonNodeInfo(item, "LOCAL_NODE_EOBJ", "local");
									NodeList.Add(CurrentNode);
									InitNodeList(ele);
								}
							}
							profileClass.classes = classes.ToArray();

							if (CurrentNode != null)
								UpdateProperties(CurrentNode);
						},
						Error = AjaxError
					});
				}
			},
			Error = AjaxError
		});
	}

	void UpdateClasses(jQueryEvent ev)
	{
		var classGroup = (JsonClassGroupInfo)ev.Data["clsgrp"];

		var info = jQuery.Select("#lv_class_group_" + classGroup.classGroupCode.ToString("X2"), ev.Target);
		if (info == null)
			return;

		if (classGroup.classes != null)
			return;

		jQuery.Ajax(new jQueryAjaxOptions()
		{
			Url = "devices/ClassList" + classGroup.classGroupCode.ToString("X2").ToUpper() + ".json",
			Success = (data, textStatus, request) =>
			{
				var content = "";
				var _classes = (dynamic[])((data.GetType() == typeof(string)) ? jQuery.ParseJson((string)data) : data);
				var classes = new List<JsonClassInfo>();
				foreach (var _item in _classes) {
					var item = new JsonClassInfo(_item);
					classes.Add(item);
					item.classGroup = classGroup;
					content += "<li data-icon=\"check\"><a href=\"#\" id=\"li_class_"
						+ classGroup.classGroupCode.ToString("X2") + item.classCode.ToString("X2") + "\">"
						+ item.description + "</a></li>";
				}
				info.Append(content).ListView().ListView("refresh");
				classGroup.classes = classes.ToArray();

				foreach (var item in classes) {
					var li = jQuery.Select("#li_class_" + classGroup.classGroupCode.ToString("X2") + item.classCode.ToString("X2"));
					li.Click(new JsDictionary("cls", item), SelectClass);
				}
			},
			Error = AjaxError
		});
	}

	void SetCurrentClass(JsonClassInfo eclass)
	{
		var item = CurrentClass;
		if (item != null) {
			jQuery.Select("#li_class_" + item.classGroup.classGroupCode.ToString("X2") + item.classCode.ToString("X2")).RemoveClass("ui-alt-icon");
		}
		CurrentClass = eclass;
		jQuery.Select("#li_class_" + eclass.classGroup.classGroupCode.ToString("X2") + eclass.classCode.ToString("X2")).AddClass("ui-alt-icon");

		jQuery.Select("#cl_class_group_" + eclass.classGroup.classGroupCode.ToString("X2")).Collapsible("expand");
	}

	void SelectClass(jQueryEvent ev)
	{
		var eclass = (JsonClassInfo)ev.Data["cls"];
		var oldclass = CurrentClass;

		var name = jQuery.Select("#ti_object_identifer")[0].GetValue();
		if (!(new Regex("^[A-Za-z_][0-9A-Za-z_]+$")).Test(name) || ((oldclass != null) && (name == oldclass.name))) {
			jQuery.Select("#ti_object_identifer")[0].SetValue(eclass.name);
		}

		SetCurrentClass(eclass);
	}

	public object AddObject(Element ele)
	{
		var eclass = CurrentClass;
		if (eclass == null) {
			NotSelectClassError();
			return false;
		}

		var name = jQuery.Select("#ti_object_identifer")[0].GetValue();
		if (!(new Regex("^[A-Za-z_][0-9A-Za-z_]+$")).Test(name))
		{
			IdentiferNotInputError();
			return false;
		}

		if (CheckIdentifer(name)) {
			InvalidIdentiferError();
			return false;
		}

		var node = CurrentNode;
		if (node == null)
			return false;

		var info = jQuery.Select("#lv_node_" + node.identifer);
		if (info == null)
			return false;

		var item = new JsonObjectInfo(eclass, name);
		item.parent = node;
		node.objects.Add(item);
		var content = "<li id=\"li_object_" + item.identifer + "\"><a href=\"#\">"
			+ item.identifer + "</a></li>";

		info.Append(content).ListView().ListView("refresh");
		var objItem = jQuery.Select("#li_object_" + item.identifer);
		jQuery.Select("a", objItem).Click(new JsDictionary("obj", item), UpdateProperties);

		var nodeItem = jQuery.Select("#cl_node_" + node.identifer);
		nodeItem.Collapsible("expand");

		return true;
	}

	public bool ChangeObject(Element ele)
	{
		var page = GetPage(ele);

		var eclass = CurrentClass;
		var obj = CurrentObject;
		if (obj == null)
			return false;

		var node = obj.parent;
		if (node == null)
			return false;

		var info = jQuery.Select("#lv_node_" + node.identifer, page);
		if (info == null)
			return false;

		var name = jQuery.Select("#ti_object_identifer", page)[0].GetValue();
		if (!(new Regex("^[A-Za-z_][0-9A-Za-z_]+$")).Test(name)) {
			IdentiferNotInputError();
			return false;
		}

		if (obj.identifer != name && CheckIdentifer(name)) {
			InvalidIdentiferError();
			return false;
		}

		var objItem = jQuery.Select("#li_object_" + obj.identifer, info);
		objItem[0].Id = "li_object_" + name;
		jQuery.Select("a", objItem).Text(name);

		info.ListView().ListView("refresh");

		obj.identifer = name;
		if ((eclass != null) && (obj.type != eclass)) {
			obj.type = eclass;
			obj.properties.Clear();
			UpdateProperties(obj);
		}

		return true;
	}

	public bool DeleteObject(Element ele)
	{
		var page = GetPage(ele);

		var obj = CurrentObject;
		if (obj == null)
			return false;

		var node = obj.parent;
		if (node == null)
			return false;

		var info = jQuery.Select("#lv_node_" + node.identifer, page);
		if (info == null)
			return false;

		jQuery.Select("#li_object_" + obj.identifer, info).Remove();
		info.ListView().ListView("refresh");

		CurrentObject = null;

		obj.parent = null;
		node.objects.Remove(obj);

		if (node.objects.Count > 0)
			CurrentObject = node.objects[0];

		if (CurrentObject != null) {
			var nodeItem = jQuery.Select("#cl_node_" + node.identifer);
			nodeItem.Collapsible("expand");

			UpdateProperties(CurrentObject);
		}

		return true;
	}

	private bool CheckIdentifer(string name)
	{
		foreach (var nod in NodeList) {
			if (nod.identifer == name) {
				return true;
			}
			foreach (var obj in nod.objects) {
				if (obj.identifer == name) {
					return true;
				}
			}
		}
		return false;
	}

	void UpdateProperties(jQueryEvent ev)
	{
		UpdateProperties((JsonObjectInfo)ev.Data["obj"]);
	}

	void UpdateProperties(JsonObjectInfo obj)
	{
		var node = obj as JsonNodeInfo;
		if (node == null) {
			node = obj.parent;
			CurrentObject = obj;
		}
		CurrentNode = node;

		jQuery.Select("#ti_node_object_name")[0].SetValue(obj.identifer);

		if (CurrentObject != null) {
			jQuery.Select("#ti_object_identifer")[0].SetValue(CurrentObject.identifer);
			SetCurrentClass(CurrentObject.type);
		}

		var eclass = obj.type;
		var info = jQuery.Select("#lv_property_list");
		if (info == null)
			return;

		if (eclass.properties != null) {
			var properties = eclass.properties;
			var content = "<li data-role=\"list-divider\">" + eclass.description + "</li>";
			foreach (var item in properties) {
				var prop = obj.FindProperty(item.propertyCode);
				content += "<li id=\"li_property_" + item.propertyCode
					+ "\" data-icon=\"check\" class=\"" + ((prop != null) ? "ui-alt-icon" : "")
					+ "\"><a href=\"#\" id=\"ak_property_" + item.propertyCode + "\">"
					+ item.description + "</a></li>";
			}
			info.Children().ReplaceWith("");
			info.Append(content).ListView().ListView("refresh");

			foreach (var item in properties) {
				jQuery.Select("#ak_property_" + item.propertyCode).Click(new JsDictionary("obj", obj, "prop", item), AddProperty);
			}
			return;
		}

		jQuery.Ajax(new jQueryAjaxOptions()
		{
			Url = "devices/PropertyList" + eclass.classGroup.classGroupCode.ToString("X2").ToUpper() + eclass.classCode.ToString("X2").ToUpper() + ".json",
			Success = (data, textStatus, request) =>
			{
				var _properties = (dynamic[])((data.GetType() == typeof(string)) ? jQuery.ParseJson((string)data) : data);
				var properties = new List<JsonPropertyInfo>();
				var content = "<li data-role=\"list-divider\">" + eclass.description + "</li>";
				foreach (var _item in _properties) {
					var item = new JsonPropertyInfo(_item);
					properties.Add(item);
					var prop = obj.FindProperty(item.propertyCode);
					content += "<li id=\"li_property_" + item.propertyCode
						+ "\" data-icon=\"check\" class=\"" + ((prop != null) ? "ui-alt-icon" : "")
						+ "\"><a href=\"#\" id=\"ak_property_" + item.propertyCode + "\">"
						+ item.description + "</a></li>";
				}
				info.Children().ReplaceWith("");
				info.Append(content).ListView().ListView("refresh");
				eclass.properties = properties.ToArray();

				foreach (var item in properties) {
					jQuery.Select("#ak_property_" + item.propertyCode).Click(new JsDictionary("obj", obj, "prop", item), AddProperty);

					if ((item.required.Count > 0) && (item.required[0] != "NONE")) {
						jQuery.Select("#li_property_" + item.propertyCode).AddClass("ui-alt-icon");
						obj.properties.Add(new JsonPropertyInfo(item));
					}
				}
			},
			Error = AjaxError
		});
	}

	void AddProperty(jQueryEvent ev)
	{
		var obj = (JsonObjectInfo)ev.Data["obj"];
		var type = (JsonPropertyInfo)ev.Data["prop"];

		var info = jQuery.Select("#li_property_" + type.propertyCode);
		if (info == null)
			return /*false*/;

		var prop = obj.FindProperty(type.propertyCode);
		if (prop == null) {
			var item = new JsonPropertyInfo(type);
			obj.properties.Add(item);

			info.AddClass("ui-alt-icon");
		}
		else {
			foreach (var item in obj.properties) {
				if (item.propertyCode != prop.propertyCode)
					continue;

				obj.properties.Remove(item);
				break;
			}

			info.RemoveClass("ui-alt-icon");
		}

		return /*true*/;
	}

	public void GenSkeleton(Element ele)
	{
		var wa = new CodeGenWorkArea();
		var macro = new StringBuilder();
		var declare = new StringBuilder();
		var predefine = new StringBuilder();
		var types = new StringBuilder();
		var funcs = new StringBuilder();
		var instance = new StringBuilder();
		var define = new StringBuilder();
		var config = new StringBuilder();

		foreach (var node in NodeList) {
			var cg = new CodeGenerator(node);
			declare.Append(cg.DeclareEchonetObject(wa));
			macro.Append(cg.EchonetObjectMacro(wa));
			predefine.Append(cg.EchonetObjectPredefine(wa));
			instance.Append(cg.DefineEchonetObject(wa));
			define.Append(cg.DefineEchonetAccesser(wa));
			config.Append(cg.ConfigEchonetObject());

			foreach (var obj in node.objects) {
				var cg1 = new CodeGenerator(obj);
				declare.Append(cg1.DeclareEchonetObject(wa));
				macro.Append(cg1.EchonetObjectMacro(wa));
				predefine.Append(cg1.EchonetObjectPredefine(wa));
				instance.Append(cg1.DefineEchonetObject(wa));
				define.Append(cg1.DefineEchonetAccesser(wa));
				config.Append(cg1.ConfigEchonetObject());
			}
		}
		types.AppendLine(CodeGenerator.DeclareTypes(wa));
		funcs.AppendLine(CodeGenerator.DeclareFunctions(wa));

		WorkArea = new JsonSkeltonResult()
		{
			macro = macro.ToString(),
			declare = declare.ToString(),
			predefine = predefine.ToString(),
			types = types.ToString(),
			funcs = funcs.ToString(),
			instance = instance.ToString(),
			define = define.ToString(),
			config = config.ToString(),
		};
		GetTemplates();
	}

	void GetTemplates()
	{
		var comp = 0;
		// main.hの生成
		jQuery.Ajax(new jQueryAjaxOptions()
		{
			Url = "templates/main.h",
			Success = (data, textStatus, request) =>
			{
				var main_h = data.ToString();
				main_h = main_h.Replace("%EchonetObjectMacro%", WorkArea.macro);
				main_h = main_h.Replace("%DeclareEchonetType%", WorkArea.types + WorkArea.declare);
				main_h = main_h.Replace("%DeclareEchonetObject%", WorkArea.predefine + WorkArea.funcs);
				MainH = main_h;
				jQuery.Select("#result_dialog_main_h").Text(MainH);
			},
			Complete = (request, textStatus) =>
			{      // 成功・失敗に関わらず通信が終了した際の処理
				comp += 1;
				if (comp == 7)
					GenSkeletonComplete();
			}
		});
		// main.cの生成
		jQuery.Ajax(new jQueryAjaxOptions()
		{
			Url = "templates/main.c",
			Success = (data, textStatus, request) =>
			{
				var main_c = data.ToString();
				main_c = main_c.Replace("%DefineEchonetObject%", WorkArea.instance + WorkArea.define);
				MainC = main_c;
				jQuery.Select("#result_dialog_main_c").Text(MainC);
			},
			Complete = (request, textStatus) =>
			{      // 成功・失敗に関わらず通信が終了した際の処理
				comp += 2;
				if (comp == 7)
					GenSkeletonComplete();
			}
		});
		// main.cfgの生成
		jQuery.Ajax(new jQueryAjaxOptions()
		{
			Url = "templates/main.cfg",
			Success = (data, textStatus, request) =>
			{
				var main_cfg = data.ToString();
				main_cfg = main_cfg.Replace("%ConfigEchonetObject%", WorkArea.config);
				MainCfg = main_cfg;
				jQuery.Select("#result_dialog_main_cfg").Text(MainCfg);
			},
			Complete = (request, textStatus) =>
			{      // 成功・失敗に関わらず通信が終了した際の処理
				comp += 4;
				if (comp == 7)
					GenSkeletonComplete();
			}
		});
	}

	void GenSkeletonComplete()
	{
		jQuery.Select("#result_dialog").Popup("open", new { transition = "slideup" });
		// ボタンを再び enableにする
		jQuery.Select("bt_gen_skeleton").Attribute("disabled", "false");
	}

	void QuerySkeleton(jQueryEvent ev)
	{
		jQuery.Select("#result_dialog_tabs").Tabs("load", 0);
	}

	private static void InvalidIdentiferError()
	{
		jQuery.Select("#popup_dialog_caption").Text("エラー");
		jQuery.Select("#popup_dialog_title").Text("ノードまたはオブジェクト名が重複しています。");
		jQuery.Select("#popup_dialog_text").Text("ノードまたはオブジェクト名は他のノードまたはオブジェクトと違う名前を付けてあげてください。");
		jQuery.Select("#popup_dialog").Popup("open", new { transition = "slidedown" });
	}

	private static void IdentiferNotInputError()
	{
		jQuery.Select("#popup_dialog_caption").Text("エラー");
		jQuery.Select("#popup_dialog_title").Text("ノードまたはオブジェクトの名前を入力してください。");
		jQuery.Select("#popup_dialog_text").Text("ノードまたはオブジェクトを追加するには、ノードまたはオブジェクトの名前を入力してください。");
		jQuery.Select("#popup_dialog").Popup("open", new { transition = "slidedown" });
	}

	private static void NotSelectClassError()
	{
		jQuery.Select("#popup_dialog_caption").Text("エラー");
		jQuery.Select("#popup_dialog_title").Text("クラスが選択されていません。");
		jQuery.Select("#popup_dialog_text").Text("オブジェクトを追加するには、上記のリストからクラスを選択してください。");
		jQuery.Select("#popup_dialog").Popup("open", new { transition = "slidedown" });
	}

	private static void AjaxError(jQueryXmlHttpRequest request, string textStatus, Exception error)
	{
		var lines = new List<string>();
		lines.Add(textStatus);
		lines.Add(((dynamic)error).message);

		var sep = new Regex("(\r\n|\r|\n)");
		string stack = ((dynamic)error).stack;
		lines.AddRange(stack.Split(sep));

		var html = jQuery.FromHtml("<p>");
		foreach (var line in lines) {
			if (sep.Exec(line) != null)
				continue;
			html.Append(Document.CreateTextNode(line).WholeText);
			html.Append(jQuery.FromHtml("<br/>"));
		}

		jQuery.Select("#popup_dialog_caption").Text("通信エラー");
		jQuery.Select("#popup_dialog_title").Text("通信エラーが発生しました");
		jQuery.Select("#popup_dialog_text").Html(html);
		jQuery.Select("#popup_dialog").Popup("open", new { transition = "slidedown" });
	}
};

public class Information
{
	public Information()
	{
	}

	public void Update(Element ele, string id)
	{
		var page = CodeGen.GetPage(ele);
		var info = jQuery.Select("#" + id, page);
		if (info == null)
			return;

		jQuery.Ajax(new jQueryAjaxOptions()
		{
			Url = "./" + id + ".json",
			Success = (data, textStatus, request) =>
			{
				var items = (dynamic[])((data.GetType() == typeof(string)) ? jQuery.ParseJson((string)data) : data);
				var html = new StringBuilder();

				html.Append("<ul data-role='listview' data-inset='true' data-theme='b'>");
				html.Append("<li data-role='list-divider'>ソフトウェア情報</li>");
				foreach (var item in items) {
					string title = item.title;
					string version = item.version;
					string link = item.link;
					string note = item.note;

					if (String.IsNullOrEmpty(link))
						html.Append("<li><a href='#'>");
					else
						html.Append("<li><a target='_blank' href='" + link + "'>");
					html.Append("<h2>" + title + "</h2>");
					html.Append("<p>Version " + version + "</p>");
					if (!String.IsNullOrEmpty(note))
						html.Append("<p>" + note + "</p>");
					html.Append("</a></li>");
				}
				html.Append("<li><p><img src='imgs/core-s.svg' /><a target='_blank' href='http://www.core-s.co.jp'>コアーズ株式会社</a></p></li>");
				html.Append("</ul>");

				info.Html(html.ToString());
				jQuery.Select("ul", info).ListView().ListView("refresh");
				info.Popup("reposition", new { positionTo = "origin" });
			},
			Error = (request, textStatus, error) =>
			{
				var html = new StringBuilder();

				html.Append("<ul data-role='listview' data-inset='true' data-theme='b'>");
				html.Append("<li data-role='list-divider'>ソフトウェア情報</li>");
				html.Append("<li><p>通信エラー");
				html.Append("</p></li>");
				html.Append("</ul>");

				info.Html(html.ToString());
				jQuery.Select("ul", info).ListView().ListView("refresh");
				info.Popup("reposition", new { positionTo = "origin" });
			}
		});
	}
}
