﻿/*
 *  TOPPERS ECHONET Lite Communication Middleware
 * 
 *  Copyright (C) 2015 Cores Co., Ltd. Japan
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id$
 */

using System;
using System.Collections.Generic;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;

public class CodeGenerator
{
	private JsonObjectInfo _ClassInfo;
	private List<JsonPropertyInfo> _PropertyInfos;

	public CodeGenerator(JsonObjectInfo cls)
	{
		_ClassInfo = cls;
		foreach (var prp in cls.properties) {
			prp.classInfo = cls;
		}
		_PropertyInfos = cls.properties;
	}

	private static string GetTypeComment(JsonFieldInfo pti)
	{
		return pti.description + "の型";
	}

	public static string GetClassIdentifier(JsonObjectInfo ci)
	{
		JsonObjectInfo cls = ci;
		string result = ci.type.type;

		if (!new Regex("^[A-Za-z_][A-Za-z0-9_]+$").Test(result)) {
			result = String.Format("ecn_cls{0:X2}{1:X2}_t", cls.type.classGroup.classGroupCode, cls.type.classCode);
		}
		else {
			var ms = new Regex("[A-Z][a-z]+", "g");
			string[] m;
			if ((m = ms.Exec(result)) != null) {
				List<string> blocks = new List<string>();
				int pos = 0;
				do {
					int len = ms.LastIndex - pos;
					blocks.Add(result.Substring(pos, len).ToLower());
					pos += len;
				} while ((m = ms.Exec(result)) != null);
				if (blocks[blocks.Count - 1] == "type") {
					blocks[blocks.Count - 1] = "t";
				}
				else {
					blocks.Add("t");
				}
				result = blocks.Join("_");
			}
		}

		return result;
	}

	private static string GetPropertyType(JsonPropertyInfo pi)
	{
		if (pi.primitive)
			return pi.type;
		else
			return "struct " + pi.type;
	}

	public static string GetPropertyIdentifier(JsonPropertyInfo pi)
	{
		JsonObjectInfo cls = pi.classInfo;
		JsonFieldInfo fi = pi;
		string result = fi.identifer;

		if (!new Regex("^[A-Za-z_][A-Za-z0-9_]+$").Test(result)) {
			result = String.Format("property{0:X2}", pi.propertyCode);
		}
		else {
			var ms = new Regex("[A-Z][a-z]+", "g");
			string[] m;
			if ((m = ms.Exec(result)) != null) {
				List<string> blocks = new List<string>();
				int pos = 0;
				do {
					int len = ms.LastIndex - pos;
					blocks.Add(result.Substring(pos, len).ToLower());
					pos += len;
				} while ((m = ms.Exec(result)) != null);
				result = blocks.Join("_");
			}
		}

		if (fi.isArray) {
			result += String.Format("[{0}]", pi.arrayCount);
		}

		return result;
	}

	private static string GetFieldType(JsonFieldInfo efi)
	{
		if (efi.primitive)
			return efi.type;
		else
			return "struct " + efi.type;
	}

	private static string GetFieldIdentifier(JsonFieldInfo efi)
	{
		JsonFieldInfo fi = efi;
		string result = fi.identifer;

		if (!new Regex("^[A-Za-z_][A-Za-z0-9_]+$").Test(result)) {
			result = String.Format("field{0:X8}", efi.type);
		}
		else {
			var ms = new Regex("[A-Z][a-z]+", "g");
			string[] m;
			if ((m = ms.Exec(result)) != null) {
				List<string> blocks = new List<string>();
				int pos = 0;
				do {
					int len = ms.LastIndex - pos;
					blocks.Add(result.Substring(pos, len).ToLower());
					pos += len;
				} while ((m = ms.Exec(result)) != null);

				result = blocks.Join("_");
			}
		}

		if (fi.isArray) {
			result += String.Format("[{0}]", efi.arrayCount);
		}

		return result;
	}

	private static string GetObjectIdentifier(JsonObjectInfo ci)
	{
		string name = ci.identifer;

		if (name.EndsWith("_EOBJ"))
			name = name.Substring(0, name.Length - 5);

		if (name.StartsWith("0x")) {
			return "eobj_" + name.Substring(2, name.Length - 2) + "_data";
		}

		return name.ToLower() + "_data";
	}

	private static string GetEobjAttribute(JsonObjectInfo eci)
	{
		switch (eci.attribute) {
		case "local":
			return "EOBJ_LOCAL_NODE";
		case "sync":
			return "EOBJ_SYNC_REMOTE_NODE";
		case "async":
			return "EOBJ_ASYNC_REMOTE_NODE";
		case "device":
			return "EOBJ_DEVICE";
		default:
			return "EOBJ_NONE";
		}
	}

	private static string GetNode(JsonObjectInfo eci)
	{
		var parent = eci.parent;

		if (parent == null)
			return "EOBJ_NULL";
		else
			return parent.identifer;
	}

	public static string GetEobjInstatceCode(JsonObjectInfo cls)
	{
		string name = cls.identifer;

		if (name.EndsWith("_EOBJ"))
			name = name.Substring(0, name.Length - 5);

		return "EOJ_X3_" + name;
	}

	private static string GetEpc(JsonPropertyInfo epi)
	{
		return String.Format("0x{0:X2}", epi.propertyCode);
	}

	private static string GetAccess(JsonPropertyInfo epi)
	{
		List<string> result = new List<string>();

		string[] access = epi.access.ToArray();

		if (access.Contains("RULE_ANNO"))
			result.Add("EPC_RULE_ANNO");

		if (access.Contains("RULE_SET"))
			result.Add("EPC_RULE_SET");

		if (access.Contains("RULE_GET"))
			result.Add("EPC_RULE_GET");

		if (access.Contains("ANNOUNCE"))
			result.Add("EPC_ANNOUNCE");

		if (access.Contains("VARIABLE"))
			result.Add("EPC_VARIABLE");

		if (result.Count == 0)
			return "EPC_NONE";

		return result.Join(" | ");
	}

	private static string GetSize(JsonPropertyInfo epi)
	{
		return (((epi.arrayCount == 0) ? 1 : epi.arrayCount) * epi.size).ToString();
	}

	private static string GetExinf(JsonPropertyInfo epi)
	{
		switch (epi.propertyCode) {
		// 現在時刻設定
		case 0x97:
		// 現在年月日設定
		case 0x98:
			return "NULL";
		// その他
		default:
			return "&" + GetObjectIdentifier(epi.classInfo) + "." + GetPropertyIdentifier(epi);
		}
	}

	private static bool HasPropSetter(JsonPropertyInfo epi, out ValueRange valueRange)
	{
		valueRange = null;

		if (!epi.primitive || (epi.arrayCount != 0))
			return false;

		valueRange = ValueRange.Parse(epi.valueDescription, epi);
		if ((valueRange.Values.Count > 0) || (valueRange.Ranges.Count > 0)) {
			return true;
		}

		valueRange = null;
		return false;
	}

	private static string GetSetter(JsonPropertyInfo epi)
	{
		switch (epi.propertyCode) {
		// 動作状態
		case 0x80:
			return "onoff_prop_set";
		// 異常発生状態
		case 0x88:
			return "alarm_prop_set";
		// 現在時刻設定
		case 0x97:
			return "time_prop_set";
		// 現在年月日設定
		case 0x98:
			return "date_prop_set";
		// その他
		default:
			ValueRange valueRange;
			if (HasPropSetter(epi, out valueRange)) {
				string clsName = GetClassIdentifier(epi.classInfo);
				if (clsName.EndsWith("_t"))
					clsName = clsName.Substring(0, clsName.Length - 2);
				return clsName + "_" + GetPropertyIdentifier(epi) + "_set";
			}
			return "ecn_data_prop_set";
		}
	}

	private static string GetSetterBody(JsonPropertyInfo epi)
	{
		StringBuilder body = new StringBuilder();

		switch (epi.propertyCode) {
		// 動作状態
		case 0x80:
			body.AppendLine("/*");
			body.AppendLine(" * 動作状態ON/OFF設定関数（0x30, 0x31のみ受け付け）");
			body.AppendLine(" */");
			body.AppendLine("int onoff_prop_set(const EPRPINIB *item, const void *src, int size, bool_t *anno)");
			body.AppendLine("{");
			body.AppendLine("	/* サイズが１以外は受け付けない */");
			body.AppendLine("	if(size != 1)");
			body.AppendLine("		return 0;");
			body.AppendLine();
			body.AppendLine("	*anno = *((uint8_t*)item->exinf) != *((uint8_t*)src);");
			body.AppendLine();
			body.AppendLine("	switch(*(uint8_t *)src){");
			body.AppendLine("	/* ONの場合 */");
			body.AppendLine("	case 0x30:");
			body.AppendLine("		*((uint8_t *)item->exinf) = *((uint8_t *)src);");
			body.AppendLine("		/* TODO: ONの場合の処理*/");
			body.AppendLine("		break;");
			body.AppendLine("	/* OFFの場合 */");
			body.AppendLine("	case 0x31:");
			body.AppendLine("		*((uint8_t *)item->exinf) = *((uint8_t *)src);");
			body.AppendLine("		/* TODO: OFFの場合の処理*/");
			body.AppendLine("		break;");
			body.AppendLine("	/* 0x30か0x31以外は受け付けない */");
			body.AppendLine("	default:");
			body.AppendLine("		return 0;");
			body.AppendLine("	}");
			body.AppendLine();
			body.AppendLine("	return 1;");
			body.AppendLine("}");
			break;
		// 異常発生状態
		case 0x88:
			body.AppendLine("/*");
			body.AppendLine(" * 異常発生状態設定関数（0x41, 0x42のみ受け付け）");
			body.AppendLine(" */");
			body.AppendLine("int alarm_prop_set(const EPRPINIB *item, const void *src, int size, bool_t *anno)");
			body.AppendLine("{");
			body.AppendLine("	ER ret;");
			body.AppendLine("	uint8_t data[1];");
			body.AppendLine("");
			body.AppendLine("	/* サイズが１以外は受け付けない */");
			body.AppendLine("	if(size != 1)");
			body.AppendLine("		return 0;");
			body.AppendLine("");
			body.AppendLine("	*anno = *((uint8_t *)item->exinf) != *((uint8_t *)src);");
			body.AppendLine("");
			body.AppendLine("	switch(*(uint8_t *)src){");
			body.AppendLine("	/* 異常発生ありの場合 */");
			body.AppendLine("	case 0x41:");
			body.AppendLine("	/* 異常発生なしの場合 */");
			body.AppendLine("	case 0x42:");
			body.AppendLine("		*((uint8_t *)item->exinf) = *((uint8_t *)src);");
			body.AppendLine("		break;");
			body.AppendLine("	/* 0x41か0x42以外は受け付けない */");
			body.AppendLine("	default:");
			body.AppendLine("		return 0;");
			body.AppendLine("	}");
			body.AppendLine("");
			body.AppendLine("	return 1;");
			body.AppendLine("}");
			break;
		// 現在時刻設定
		case 0x97:
			body.AppendLine("/*");
			body.AppendLine(" * 現在時刻設定関数");
			body.AppendLine(" */");
			body.AppendLine("int time_prop_set(const EPRPINIB *item, const void *src, int size, bool_t *anno)");
			body.AppendLine("{");
			body.AppendLine("	uint8_t *p_src;");
			body.AppendLine();
			body.AppendLine("	if(size != 2)");
			body.AppendLine("		return 0;");
			body.AppendLine();
			body.AppendLine("	/* 書き込み終了確認 */");
			body.AppendLine("	while((sil_reb_mem(RC1CC2) & RC1CC2_RWST) != 0)");
			body.AppendLine("		tslp_tsk(1);");
			body.AppendLine();
			body.AppendLine("	/* カウンタ停止設定 */");
			body.AppendLine("	sil_wrb_mem(RC1CC2, sil_reb_mem(RC1CC2) | RC1CC2_RWAIT);");
			body.AppendLine();
			body.AppendLine("	/* カウンタのウエイト状態の確認 */");
			body.AppendLine("	while((sil_reb_mem(RC1CC2) & RC1CC2_RWST) == 0)");
			body.AppendLine("		tslp_tsk(1);");
			body.AppendLine();
			body.AppendLine("	/* 時刻設定 */");
			body.AppendLine("	p_src = (uint8_t *)src;");
			body.AppendLine("	sil_wrb_mem(RC1HOUR, *p_src++);");
			body.AppendLine("	sil_wrb_mem(RC1MIN, *p_src++);");
			body.AppendLine("	sil_wrb_mem(RC1SEC, 0x00);");
			body.AppendLine();
			body.AppendLine("	/* カウンタ動作設定 */");
			body.AppendLine("	sil_wrb_mem(RC1CC2, sil_reb_mem(RC1CC2) & ~RC1CC2_RWAIT);");
			body.AppendLine();
			body.AppendLine("	return (intptr_t)p_src - (intptr_t)src;");
			body.AppendLine("}");
			break;
		// 現在年月日設定
		case 0x98:
			body.AppendLine("/*");
			body.AppendLine(" * 現在年月日設定関数");
			body.AppendLine(" */");
			body.AppendLine("int date_prop_set(const EPRPINIB *item, const void *src, int size, bool_t *anno)");
			body.AppendLine("{");
			body.AppendLine("	uint8_t *p_src;");
			body.AppendLine();
			body.AppendLine("	if(size != 4)");
			body.AppendLine("		return 0;");
			body.AppendLine();
			body.AppendLine("	/* 書き込み終了確認 */");
			body.AppendLine("	while((sil_reb_mem(RC1CC2) & RC1CC2_RWST) != 0)");
			body.AppendLine("		tslp_tsk(1);");
			body.AppendLine();
			body.AppendLine("	/* カウンタ停止設定 */");
			body.AppendLine("	sil_wrb_mem(RC1CC2, sil_reb_mem(RC1CC2) | RC1CC2_RWAIT);");
			body.AppendLine();
			body.AppendLine("	/* カウンタのウエイト状態の確認 */");
			body.AppendLine("	while((sil_reb_mem(RC1CC2) & RC1CC2_RWST) == 0)");
			body.AppendLine("		tslp_tsk(1);");
			body.AppendLine();
			body.AppendLine("	/* 年月日設定 */");
			body.AppendLine("	p_src = (uint8_t *)src;");
			body.AppendLine("	p_src++;	/* 20XX */");
			body.AppendLine("	sil_wrb_mem(RC1YEAR, *p_src++);");
			body.AppendLine("	sil_wrb_mem(RC1MONTH, *p_src++);");
			body.AppendLine("	sil_wrb_mem(RC1DAY, *p_src++);");
			body.AppendLine();
			body.AppendLine("	/* カウンタ動作設定 */");
			body.AppendLine("	sil_wrb_mem(RC1CC2, sil_reb_mem(RC1CC2) & ~RC1CC2_RWAIT);");
			body.AppendLine();
			body.AppendLine("	return (intptr_t)p_src - (intptr_t)src;");
			body.AppendLine("}");
			break;
		// その他
		default:
			ValueRange valueRange;
			if (HasPropSetter(epi, out valueRange)) {
				string Xn = "X" + (2 * epi.size).ToString();
				string indent = "	";

				body.AppendLine("/*");
				body.AppendLine(String.Format(" * {0}設定関数", epi.description));
				body.AppendLine(" */");
				body.AppendLine(String.Format("int {0}(const EPRPINIB *item, const void *src, int size, bool_t *anno)", GetSetter(epi)));
				body.AppendLine("{");
				body.AppendLine(String.Format("	/* サイズが{0}以外は受け付けない */", epi.size));
				body.AppendLine(String.Format("	if(size != {0})", epi.size));
				body.AppendLine("		return 0;");
				body.AppendLine();
				if (valueRange.Values.Count > 0) {
					indent += "	";
					body.AppendLine(String.Format("	switch(*({0} *)src){{", epi.type));
					foreach (var i in valueRange.Values) {
						body.AppendLine(String.Format("	/* {0}の場合 */", i.Disp));
						body.AppendLine(String.Format("	case 0x{0}:", i.Val.ToString(16)));
						body.AppendLine(String.Format("		*(({0} *)item->exinf) = *(({0} *)src);", epi.type));
						body.AppendLine(String.Format("		/* TODO: {0}の場合の処理*/", i.Disp));
						body.AppendLine("		break;");
					}
					body.AppendLine("	default:");
				}
				bool first = true;
				foreach (var i in valueRange.Ranges) {
					if (!String.IsNullOrEmpty(i.Disp))
						body.AppendLine(String.Format(indent + "/* {0}の場合 */", i.Disp));
					if (first) {
						first = false;
						body.Append(indent);
					}
					else {
						body.Append(indent + "else ");
					}
					body.AppendLine(String.Format("if((*({0} *)src >= 0x{1}) && (*({0} *)src <= 0x{2})){{", epi.type, i.Min.ToString(16), i.Max.ToString(16)));
					body.AppendLine(String.Format(indent + "	*(({0} *)item->exinf) = *(({0} *)src);", epi.type));
					body.AppendLine(String.Format(indent + "	/* TODO: {0}の場合の処理*/", String.IsNullOrEmpty(i.Disp) ? "この" : i.Disp));
					body.AppendLine(indent + "}");
				}
				if (valueRange.Values.Count > 0) {
					body.AppendLine("		/* 上記以外は受け付けない */");
					body.AppendLine("		return 0;");
					body.AppendLine("	}");
				}
				else {
					body.AppendLine("	/* 上記以外は受け付けない */");
					body.AppendLine("	else{");
					body.AppendLine("		return 0;");
					body.AppendLine("	}");
				}
				body.AppendLine();
				body.AppendLine(String.Format("	return {0};", epi.size));
				body.AppendLine("}");
				break;
			}
			break;
		}

		body.AppendLine();

		return body.ToString();
	}

	private static string GetGetter(JsonPropertyInfo epi)
	{
		switch (epi.propertyCode) {
		// 現在時刻設定
		case 0x97:
			return "time_prop_get";
		// 現在年月日設定
		case 0x98:
			return "date_prop_get";
		// その他
		default:
			return "ecn_data_prop_get";
		}
	}

	private static string GetGetterBody(JsonPropertyInfo epi)
	{
		StringBuilder body = new StringBuilder();

		switch (epi.propertyCode) {
		// 現在時刻設定
		case 0x97:
			body.AppendLine("/*");
			body.AppendLine(" * 現在時刻取得関数");
			body.AppendLine(" */");
			body.AppendLine("int time_prop_get(const EPRPINIB *item, void *dst, int size)");
			body.AppendLine("{");
			body.AppendLine("	uint8_t *p_dst;");
			body.AppendLine();
			body.AppendLine("	if(size != 2)");
			body.AppendLine("		return 0;");
			body.AppendLine();
			body.AppendLine("	/* 読み出し終了確認 */");
			body.AppendLine("	while((sil_reb_mem(RC1CC2) & RC1CC2_RWST) != 0)");
			body.AppendLine("		tslp_tsk(1);");
			body.AppendLine();
			body.AppendLine("	/* カウンタ停止設定 */");
			body.AppendLine("	sil_wrb_mem(RC1CC2, sil_reb_mem(RC1CC2) | RC1CC2_RWAIT);");
			body.AppendLine();
			body.AppendLine("	/* カウンタのウエイト状態の確認 */");
			body.AppendLine("	while((sil_reb_mem(RC1CC2) & RC1CC2_RWST) == 0)");
			body.AppendLine("		tslp_tsk(1);");
			body.AppendLine();
			body.AppendLine("	/* 時刻設定 */");
			body.AppendLine("	p_dst = (uint8_t *)dst;");
			body.AppendLine("	*p_dst++ = sil_reb_mem(RC1HOUR);");
			body.AppendLine("	*p_dst++ = sil_reb_mem(RC1MIN);");
			body.AppendLine();
			body.AppendLine("	/* カウンタ動作設定 */");
			body.AppendLine("	sil_wrb_mem(RC1CC2, sil_reb_mem(RC1CC2) & ~RC1CC2_RWAIT);");
			body.AppendLine();
			body.AppendLine("	return (intptr_t)p_dst - (intptr_t)dst;");
			body.AppendLine("}");
			break;
		// 現在年月日設定
		case 0x98:
			body.AppendLine("/*");
			body.AppendLine(" * 現在年月日取得関数");
			body.AppendLine(" */");
			body.AppendLine("int date_prop_get(const EPRPINIB *item, void *dst, int size)");
			body.AppendLine("{");
			body.AppendLine("	uint8_t *p_dst;");
			body.AppendLine();
			body.AppendLine("	if(size != 4)");
			body.AppendLine("		return 0;");
			body.AppendLine();
			body.AppendLine("	/* 読み出し終了確認 */");
			body.AppendLine("	while((sil_reb_mem(RC1CC2) & RC1CC2_RWST) != 0)");
			body.AppendLine("		tslp_tsk(1);");
			body.AppendLine();
			body.AppendLine("	/* カウンタ停止設定 */");
			body.AppendLine("	sil_wrb_mem(RC1CC2, sil_reb_mem(RC1CC2) | RC1CC2_RWAIT);");
			body.AppendLine();
			body.AppendLine("	/* カウンタのウエイト状態の確認 */");
			body.AppendLine("	while((sil_reb_mem(RC1CC2) & RC1CC2_RWST) == 0)");
			body.AppendLine("		tslp_tsk(1);");
			body.AppendLine();
			body.AppendLine("	p_dst = (uint8_t *)dst;");
			body.AppendLine("	*p_dst++ = 0x20;");
			body.AppendLine("	*p_dst++ = sil_reb_mem(RC1YEAR);");
			body.AppendLine("	*p_dst++ = sil_reb_mem(RC1MONTH);");
			body.AppendLine("	*p_dst++ = sil_reb_mem(RC1DAY);");
			body.AppendLine();
			body.AppendLine("	/* カウンタ動作設定 */");
			body.AppendLine("	sil_wrb_mem(RC1CC2, sil_reb_mem(RC1CC2) & ~RC1CC2_RWAIT);");
			body.AppendLine();
			body.AppendLine("	return (intptr_t)p_dst - (intptr_t)dst;");
			body.AppendLine("}");
			break;
		// その他
		default:
			break;
		}

		body.AppendLine();

		return body.ToString();
	}

	private static void GetInitialValue(StringBuilder body, JsonFieldInfo emi, string description, string valRng, string indent)
	{
		if (emi.type == "manufacturer_code_t") {
			body.AppendLine(indent + "{ MAKER_CODE },\t/* メーカーコード */");
			return;
		}

		if (emi.type == "version_information_t") {
			body.AppendLine(indent + "{ 0x01, 0x0A, 0x01, 0x00 },\t/* Ｖｅｒｓｉｏｎ情報 */");
			return;
		}

		if (emi.type == "standard_version_information_t") {
			body.AppendLine(indent + "{ 0x00, 0x00, 'C', 0x00 },\t/* 規格Ｖｅｒｓｉｏｎ情報 */");
			return;
		}

		if (emi.primitive) {
			int count = emi.arrayCount;
			if (count == 0) {
				body.AppendLine(String.Format(indent + "{0},\t/* {1} */", GetInitialValue(valRng, emi), description));
			}
			else {
				body.Append(indent + "{ ");
				for (int i = 0; i < count; i++) {
					body.Append(String.Format("{0}, ", GetInitialValue(valRng, emi)));
				}
				body.AppendLine(String.Format("}},\t/* {0} */", description));
			}
		}
		else {
			body.AppendLine(indent + "{");
			foreach (var efi in emi.fields) {
				GetInitialValue(body, efi, efi.description, efi.valueDescription, indent + "\t");
			}
			body.AppendLine(indent + "},");
		}
	}

	private static string GetInitialValue(string valRng, JsonFieldInfo pi)
	{
		ValueRange valueRange = ValueRange.Parse(valRng, pi);
		string init = "0000000000000000" + valueRange.InitailValue;

		char[] temp = new char[2 + 2 * pi.size];
		temp[0] = '0';
		temp[1] = 'x';
		for (int i = 2, j = init.Length - 2 * pi.size; i < temp.Length; i++, j++) {
			temp[i] = init.CharAt(j);
		}
		return new String(temp);
	}

	private static bool IsExtractProperty(JsonPropertyInfo prp)
	{
		switch (prp.propertyCode) {
		case 0x9B:	// ＳｅｔＭプロパティマップ
		case 0x9C:	// ＧｅｔＭプロパティマップ
			// はECHONET Liteにはないので作らない
			return true;
		case 0x9D:	// 状変アナウンスプロパティマップ
		case 0x9E:	// Ｓｅｔプロパティマップ
		case 0x9F:	// Ｇｅｔプロパティマップ
		case 0xD3:	// 自ノードインスタンス数
		case 0xD4:	// 自ノードクラス数
		case 0xD5:	// インスタンスリスト通知
		case 0xD6:	// 自ノードインスタンスリストＳ
		case 0xD7:	// 自ノードクラスリストＳ
			// はコンフィグレーターで生成するので作らない
			return true;
		}

		return false;
	}

	public string DeclareEchonetObject(CodeGenWorkArea wa)
	{
		StringBuilder body = new StringBuilder();

		body.AppendLine(String.Format("/*\r\n *  {0}\r\n */", _ClassInfo.type.description));
		body.AppendLine(String.Format("struct {0} {{", GetClassIdentifier(_ClassInfo)));
		foreach (JsonPropertyInfo pi in _PropertyInfos) {
			JsonPropertyInfo prp = pi;
			if (IsExtractProperty(prp))
				continue;

			JsonFieldInfo pti = pi;
			if (!pti.primitive && (FirstOrDefault(wa.Types, pti) == null)) {
				wa.Types.Add(pti);
			}

			string[] funcpair = null;
			string func = GetSetter(pi);
			if ((func != "ecn_data_prop_set") && !wa.SetterDecs.Contains(func)) {
				wa.SetterDecs.Add(func);
				if (funcpair == null)
					funcpair = new string[2];
				funcpair[0] = func;
			}

			func = GetGetter(pi);
			if ((func != "ecn_data_prop_get") && !wa.GetterDecs.Contains(func)) {
				wa.GetterDecs.Add(func);
				if (funcpair == null)
					funcpair = new string[2];
				funcpair[1] = func;
			}

			if (funcpair != null) {
				wa.Functions.Add(funcpair);
			}

			body.AppendLine(String.Format("\t/* {0} */", prp.description));
			body.AppendLine(String.Format("\t{0} {1};", GetPropertyType(pi), GetPropertyIdentifier(pi)));
		}
		body.AppendLine(String.Format("}};"));
		body.AppendLine();

		return body.ToString();
	}

	private JsonFieldInfo FirstOrDefault(List<JsonFieldInfo> list, JsonFieldInfo pti)
	{
		foreach (JsonFieldInfo p in list) {
			if (p.type == pti.type)
				return p;
		}
		return null;
	}

	public string EchonetObjectMacro(CodeGenWorkArea wa)
	{
		StringBuilder body = new StringBuilder();

		body.AppendLine(String.Format("#define {0} {1}\t/* {2}のインスタンスコード */", GetEobjInstatceCode(_ClassInfo), _ClassInfo.instatceCode, _ClassInfo.type.description));
		body.AppendLine();

		return body.ToString();
	}

	public string EchonetObjectPredefine(CodeGenWorkArea wa)
	{
		StringBuilder body = new StringBuilder();

		body.AppendLine(String.Format("extern struct {0} {1};\t/* {2}のデータ */", GetClassIdentifier(_ClassInfo), GetObjectIdentifier(_ClassInfo), _ClassInfo.type.description));
		body.AppendLine();

		return body.ToString();
	}

	public static string DeclareTypes(CodeGenWorkArea wa)
	{
		StringBuilder body = new StringBuilder();

		foreach (JsonFieldInfo pti in wa.Types) {
			body.AppendLine(String.Format("/*\r\n *  {0}\r\n */", GetTypeComment(pti)));
			body.AppendLine(String.Format("struct {0} {{", pti.type));
			foreach (JsonFieldInfo efi in pti.fields) {
				body.AppendLine(String.Format("\t/* {0} */", efi.description));
				body.AppendLine(String.Format("\t{0} {1};", GetFieldType(efi), GetFieldIdentifier(efi)));
			}
			body.AppendLine(String.Format("}};"));
			body.AppendLine();
		}

		return body.ToString();
	}

	public static string DeclareFunctions(CodeGenWorkArea wa)
	{
		StringBuilder body = new StringBuilder();

		foreach (string[] funcpair in wa.Functions) {
			if (funcpair[0] != null)
				body.AppendLine(String.Format("EPRP_SETTER {0};", funcpair[0]));
			if (funcpair[1] != null)
				body.AppendLine(String.Format("EPRP_GETTER {0};", funcpair[1]));
		}

		return body.ToString();
	}

	public string DefineEchonetObject(CodeGenWorkArea wa)
	{
		StringBuilder body = new StringBuilder();

		string indent = "\t";
		body.AppendLine(String.Format("/* {0} */", _ClassInfo.type.description));
		body.AppendLine(String.Format("struct {0} {1} = {{", GetClassIdentifier(_ClassInfo), GetObjectIdentifier(_ClassInfo)));
		foreach (JsonPropertyInfo pi in _PropertyInfos) {
			JsonPropertyInfo prp = pi;
			if (IsExtractProperty(prp))
				continue;

			GetInitialValue(body, pi, prp.description, prp.valueDescription, indent);
		}
		body.AppendLine("};");
		body.AppendLine();

		return body.ToString();
	}

	public string DefineEchonetAccesser(CodeGenWorkArea wa)
	{
		StringBuilder body = new StringBuilder();

		foreach (JsonPropertyInfo pi in _PropertyInfos) {
			JsonPropertyInfo prp = pi;
			if (IsExtractProperty(prp))
				continue;

			string func = GetSetter(pi);
			if (!wa.SetterDefs.Contains(func)) {
				wa.SetterDefs.Add(func);
				body.Append(GetSetterBody(pi));
			}

			func = GetGetter(pi);
			if (!wa.GetterDefs.Contains(func)) {
				wa.GetterDefs.Add(func);
				body.Append(GetGetterBody(pi));
			}
		}

		return body.ToString();
	}

	public string ConfigEchonetObject()
	{
		StringBuilder body = new StringBuilder();

		body.AppendLine(String.Format("/*\r\n *  {0}\r\n */", _ClassInfo.type.description));
		body.AppendLine(String.Format("ECN_CRE_EOBJ ({0}, {{ {1}, {2}, 0, {3}, {4}, {5} }});",
			_ClassInfo.identifer, GetEobjAttribute(_ClassInfo), GetNode(_ClassInfo), _ClassInfo.type.classGroup.define, _ClassInfo.type.define, GetEobjInstatceCode(_ClassInfo)));
		body.AppendLine();
		foreach (JsonPropertyInfo epi in _PropertyInfos) {
			JsonPropertyInfo prp = epi;
			if (IsExtractProperty(prp))
				continue;

			body.AppendLine(String.Format("/* {0} */", prp.description));
			body.AppendLine(String.Format("ECN_DEF_EPRP ({0}, {{ {1}, {2}, {3}, (intptr_t){4}, (EPRP_SETTER *){5}, (EPRP_GETTER *){6} }});",
				_ClassInfo.identifer, GetEpc(epi), GetAccess(epi), GetSize(epi), GetExinf(epi), GetSetter(epi), GetGetter(epi)));
			body.AppendLine();
		}

		return body.ToString();
	}
}

public class CodeGenWorkArea
{
	public List<JsonFieldInfo> Types = new List<JsonFieldInfo>();
	public List<string> SetterDecs = new List<string>();
	public List<string> GetterDecs = new List<string>();
	public List<string[]> Functions = new List<string[]>();
	public List<string> SetterDefs = new List<string>();
	public List<string> GetterDefs = new List<string>();
}
