﻿/*
 *  TOPPERS ECHONET Lite Communication Middleware
 * 
 *  Copyright (C) 2015 Cores Co., Ltd. Japan
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id$
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using System.Net;

namespace EcnlSklGen
{
	public partial class MainForm : Form
	{
		IntPtr hMod;

		public MainForm()
		{
			hMod = LoadLibraryEx(@"EcnlSklGenRes.dll", IntPtr.Zero, LOAD_LIBRARY_AS_DATAFILE);
			InitializeComponent();
			webBrowser1.ObjectForScripting = new ScriptingHost(this);
		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			webBrowser1.DocumentTitleChanged += new EventHandler(webBrowser1_DocumentTitleChanged);
		}

		private void webBrowser1_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			int zoom = (int)(Screen.FromControl(this).Bounds.Height / 9.60);
			if (zoom > 100)
				zoom = 100;
			((dynamic)webBrowser1.ActiveXInstance).ExecWB(63, 1, zoom, 0);

			Icon favicon = GetFavicon(e.Url);
			if (favicon != null)
				Icon = favicon;
		}

		private void webBrowser1_DocumentTitleChanged(object sender, EventArgs e)
		{
			string text = webBrowser1.DocumentTitle;
			if (text != "")
				Text = text;
		}

		public Icon GetFavicon(Uri url)
		{
			if (url.Scheme == "res") {
				byte[] iconbuf;
				if (GetResource("favicon.ico", out iconbuf))
					return new Icon(new MemoryStream(iconbuf));
			}
			else {
				string iconurl = url.Scheme + "://" + url.Host + "/favicon.ico";
				WebRequest request = WebRequest.Create(iconurl);
				try {
					WebResponse response = request.GetResponse();

					return new Icon(response.GetResponseStream());
				}
				catch (Exception) {
				}
			}

			return null;
		}

		[DllImport("kernel32.dll", SetLastError = true)]
		static extern IntPtr LoadLibraryEx(string lpFileName, IntPtr hFile, uint dwFlags);
		[DllImport("kernel32.dll", SetLastError = true)]
		static extern IntPtr FindResource(IntPtr hModule, string lpName, string lpType);
		[DllImport("kernel32.dll")]
		static extern IntPtr FindResource(IntPtr hModule, string lpName, uint lpType);
		[DllImport("kernel32.dll", SetLastError = true)]
		static extern IntPtr LoadResource(IntPtr hModule, IntPtr hResInfo);
		[DllImport("kernel32.dll", SetLastError = true)]
		static extern uint SizeofResource(IntPtr hModule, IntPtr hResInfo);

		const uint LOAD_LIBRARY_AS_DATAFILE = 0x00000002;

		internal bool GetResource(string url, out byte[] bPtr)
		{
			bPtr = null;

			if (hMod == IntPtr.Zero)
				return false;

			IntPtr hRes = FindResource(hMod, url, 23);
			if (hRes == IntPtr.Zero)
				return false;

			uint size = SizeofResource(hMod, hRes);
			IntPtr pt = LoadResource(hMod, hRes);

			bPtr = new byte[size];
			Marshal.Copy(pt, bPtr, 0, (int)size);

			return true;
		}
	}

	[System.Runtime.InteropServices.ComVisible(true)]
	public class ScriptingHost
	{
		MainForm _form;
		WebBrowserReadyState _readyState;
		int _status;
		string _type;
		string _url;
		bool _async;
		byte[] _response;
		string _responseText;
		string _contentType;

		public ScriptingHost(MainForm form)
		{
			_form = form;
		}

		public object CreateXMLHttpRequest()
		{
			return new ScriptingHost(_form);
		}

		public void open(string type, string url, bool async)
		{
			open(type, url, async, "", "");
		}

		public void open(string type, string url, bool async, string username, string password)
		{
			_type = type;
			_url = url;
			_async = async;

			_status = 404;
			_readyState = WebBrowserReadyState.Loading;
			if (onreadystatechange != null)
				onreadystatechange.call(null, true);
		}

		public void send(object body)
		{
			_readyState = WebBrowserReadyState.Loaded;
			if (onreadystatechange != null)
				onreadystatechange.call(null, false);

			if (_async)
				_form.BeginInvoke(new MethodInvoker(() => { Load(_url); }));
			else
				Load(_url);
		}

		public string getAllResponseHeaders()
		{
			return
				"Content-Type: " + _contentType + "\n" +
				"Last-Modified: Tue, 11 Nov 2014 00:00:00 GMT\n" +
				"Accept-Ranges: bytes\n" +
				"Server: Microsoft-IIS/8.0\n" +
				"Date: " + DateTime.Now.ToString("R");
		}

		public int readyState { get { return (int)_readyState; } }
		public int status { get { return _status; } }
		public string statusText { get { return (_status == 200) ? "OK" : "Not Found"; } }
		public string responseText { get { return _responseText; } }
		public dynamic onreadystatechange { get; set; }

		private void Load(string url)
		{
			if (url.StartsWith("./"))
				url = url.Substring(2);

			if (!_form.GetResource(url, out _response)) {
				_status = 404;
				_readyState = WebBrowserReadyState.Complete;
				if (onreadystatechange != null)
					onreadystatechange.call(null, true);
				return;
			}

			switch (System.IO.Path.GetExtension(url)) {
			case "html": _contentType = "text/html"; break;
			case "css": _contentType = "text/css"; break;
			case "js": _contentType = "text/javascript"; break;
			case "gif": _contentType = "image/gif"; break;
			case "jpeg": _contentType = "image/jpeg"; break;
			case "png": _contentType = "image/png"; break;
			case "svg": _contentType = "image/svg+xml"; break;
			case "json": _contentType = "application/json"; break;
			default: _contentType = "text/plane"; break;
			}
			_responseText = Encoding.UTF8.GetString(_response);

			_status = 200;
			_readyState = WebBrowserReadyState.Complete;
			if (onreadystatechange != null)
				onreadystatechange.call(null, false);
		}
	}
}
